-- classes.e

include myLibs\myDebug.e

-- a class is a template
-- it is refered to by a name
-- it contains values and methods (functions)
-- to use a class, you must create entities based on the class

-- to manage classes and entities, we use 2 sequences
--
-- the first one is named classes. It lists classes.
--   each class item consists of 2 sequences: {name, methods}
--     name is the name of the class
--     methods is a sequence listing all methods in form of pairs {method name, routine id}
--   there is only one method to manage classes
--     procedure defineClass(name, methods) to define a class
--
-- the second one is named entities. It lists entities.
--   each entity item consists of 3 sequences: {name, class, values}
--     name is the name of the entity
--     class is the base class name
--     values is a sequence listing all values contained in the entity
--   there are 3 methods to manage entities
--     procedure createEntity(sequence name, sequence class, sequence values) to create an entity
--       name is the entity's name
--       class is the base class' name
--       values is a sequence of initial values of all contained variables
--     procedure removeEntity(sequence name) to remove an entity
--       name is the name of the entity
--     function method(sequence entity, sequence name, sequence params) to call a method (function) within the entity
--       entity is the name of the entity
--       name is the method's name
--       params is a sequence containing all the method parameters
--       the function returns the result of the method

------------------------------------------------------------------------------
-- INTERNAL CONSTANTS
--
-- leave this section as is
------------------------------------------------------------------------------

constant CLASS_NAME =1, CLASS_METHODS=2
constant METHOD_NAME=1, METHOD_ID=2
global constant ENTITY_NAME=1, ENTITY_CLASS=2, ENTITY_VALUES=3

------------------------------------------------------------------------------
-- INTERNAL VARIABLES
--
-- leave this section as is
------------------------------------------------------------------------------

sequence classes
classes = {}

global sequence entities
entities = {}

------------------------------------------------------------------------------
-- INTERNAL FUNCTIONS
--
-- leave this section as is
------------------------------------------------------------------------------

function findClass(sequence name)
  for i = 1 to length(classes) do
    if compare(classes[i][CLASS_NAME], name) = 0 then
      return i
    end if
  end for
  return 0
end function

------------------------------------------------------------------------------

function findMethod(sequence class, sequence name)
  integer cl
  sequence meth
  
  cl = findClass(class)
  for i = 1 to length(classes[cl][CLASS_METHODS]) do
    meth = classes[cl][CLASS_METHODS][i]
    if compare(meth[METHOD_NAME], name) = 0 then
      return meth[METHOD_ID]
    end if
  end for
  return 0
end function

------------------------------------------------------------------------------

function findEntity(sequence name)
  for i = 1 to length(entities) do
    if compare(entities[i][ENTITY_NAME], name) = 0 then
      return i
    end if
  end for
  return 0
end function

include user.e

------------------------------------------------------------------------------
-- GLOBAL METHODS
--
-- leave this section as is
------------------------------------------------------------------------------

-- define a class
global procedure defineClass(sequence name, sequence methods)
  sequence s

  s = {}
  for i = 1 to length(methods) do
    s = append(s, {methods[i],routine_id(methods[i])})
  end for
  classes = append(classes, {name, s})
end procedure

------------------------------------------------------------------------------

-- create an entity of the class and preset its values
-- doesn't care which type the value is: you should know
global procedure createEntity(sequence name, sequence class, sequence values)
  entities = append(entities, {name, class, values})
end procedure

------------------------------------------------------------------------------

-- remove en entity
global procedure removeEntity(sequence name)
  integer en
  
  en = findEntity(name)
  if en = 0 then return end if
  if en = 1 then
    entities = entities[2..$]
  elsif en = length(entities) then
    entities = entities[1..$-1]
  else
    entities = entities[1..en-1] & entities[en+1..$]
  end if
end procedure

------------------------------------------------------------------------------

-- call a method (function) within an entity
global function method(sequence entity, sequence name, sequence params)
  integer en, id
  sequence cl
  
  en = findEntity(entity)
  if en = 0 then
    printf(f_debug, "entity %s not found\n", entity)
    return -1
  end if
  cl = entities[en][ENTITY_CLASS]
  id = findMethod(cl, name)
  if id = -1 then
    printf(f_debug, "method %s not found\n", name)
    return -1
  end if
  return call_func(id, {en, params})
end function

